#include <stdio.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>  /* Per i socket AF_UNIX */
#include <sys/stat.h>
#include <fcntl.h>

#include "common.h"

#define ERR_MESG   "Ricetta Inesistente!"
#define TOT_REC 3   

/* scrive la ricetta 'recNum' per il client sul socket 'fd' */
int writeRecipe (int fd, int recNum);

int main (void) {
  int serverFd, clientFd, serverLen, clientLen, recNum;

  struct sockaddr_un serverUNIXAddress;  /* Server address */
  struct sockaddr_un clientUNIXAddress;  /* Client address */
  struct sockaddr* serverSockAddrPtr;    /* Ptr to server address */
  struct sockaddr* clientSockAddrPtr;    /* Ptr to client address */

  /* Ignora il segnale SIGCHLD per evitare la generazione di zombie */
  /* Nota: E` diverso dall'ignorare il segnale stabilito per default */
  signal (SIGCHLD, SIG_IGN);

  serverSockAddrPtr = (struct sockaddr*) &serverUNIXAddress;
  serverLen = sizeof (serverUNIXAddress);
  clientSockAddrPtr = (struct sockaddr*) &clientUNIXAddress;
  clientLen = sizeof (clientUNIXAddress);

  /* Crea un socket UNIX, bidirezionale, protocollo di default */
  serverFd = socket (AF_UNIX, SOCK_STREAM, DEFAULT_PROTOCOL);
  serverUNIXAddress.sun_family = AF_UNIX; /* Dominio */
  strcpy (serverUNIXAddress.sun_path, "recipe"); /* Nome */
  unlink ("recipe"); /* Cancella il file se esiste gia` */
  bind (serverFd, serverSockAddrPtr, serverLen); /* Crea il file */
  listen (serverFd, 5); /* Massimo numero di connessioni pendenti */

  while (1) {
    /* Attende una connessione */
     clientFd = accept (serverFd, clientSockAddrPtr, &clientLen);
     if (fork () == 0) { /* Crea un figlio per servire la richiesta */
       read(clientFd, &recNum, sizeof(int)); /* Legge la ricetta desiderata */
       writeRecipe (clientFd, recNum); /* ... e la fornisce. */
       close (clientFd); /* Chiude il socket */
       exit (EXIT_SUCCESS);
     } 
     else /* padre */
       close (clientFd); /* Chiude il  client descriptor */
  }
}


/* scrive la ricetta 'recNum' per il client sul socket 'fd' */
int writeRecipe (int fd, int recNum) {
  int recFd; /* descrittore del file della ricetta */

  /* array dei nomi dei file dove sono contenute le ricette */
  static char *recipes[TOT_REC] = { "pasta_amatriciana.rec",
				    "pollo_alla_curda.rec",
				    "involtini_armeni.rec"};

  char str[MAX_LINE];        /* per tenere righe di testo */

  if (recNum < TOT_REC) {
    /* Scrive la ricetta su fd */
    recFd = open(recipes[recNum], O_RDONLY);
    while (readLine(recFd, '\n', str))  /* legge le ricette dal file. 
					   Terminatore di riga \n */
      write (fd, str, strlen(str) + 1);
    return 1;
  }
  else {
    /* Errore */
    write (fd, ERR_MESG, strlen(ERR_MESG) + 1);
    return -1;
  }
}
